/******************************************************************************************************************
*	Author:				Jacquelin POTIER
*	Date:				10/31/2002
*	Copyright:			 2002, Jacquelin POTIER (jacquelin.potier@libertysurf.fr). 
*                                       Free for non commercial use (please let author's name)
/******************************************************************************************************************/
using System;
using System.Threading;

	/// <summary>
	/// Description rsume de class_agent.
	/// </summary>
	/// 
	public class class_agent
	{

		#region const
		
		private const byte BalloonOn=1;

		#endregion

		#region members

        // Notice : .Net support of agent limited to 3.5 an 4.0 version by double agent. 
        //          This implies program crash if this application is built in .Net 1.0, 1.1 and 2.0 with double agent
        //          => Build for win 8 and upper must be done in .Net 3.5 or .Net 4.0
        // may to be changed by using a cpp wrapper
		public	AgentObjects.Agent						ctl_agent;
		public  AgentObjects.IAgentCtlCharacterEx		obj_char_ex;
		private AgentObjects.IAgentCtlRequest			hide_request;
		private AgentObjects.IAgentCtlRequest			speak_request;

		private bool									agent_unloaded;
		private string									unload_agent_key;
		private bool									agent_being_unlading;
		private bool									agent_being_speaking;
			
		public string									chrtag; //Sets the character of the voice
		//"Normal"    (Default) A normal tone of voice.
		//"Monotone"    A monotone voice.
		//"Whisper"    A whispered voice.
		public string									ctxtag; //Sets the context of the output text
		//"Address"    Addresses and/or phone numbers.
		//"E-mail"    Electronic mail.
		//"Unknown"    (Default) Context is unknown.
		public int										pittag; //Sets the baseline pitch of the output to the specified value in hertz
		//50 <pit <250
		public int										spdtag; //Sets the baseline average talking speed of the speech output,in words per minute
		//50 <speed <250
		public bool										default_voice; // if true don't use chrtag,ctxtag,pittag,spdtag
		
		public bool										speak_stop_actions;

		#endregion

		#region constructor

		public class_agent()
		{
			ctl_agent=new AgentObjects.Agent();
			
			ctl_agent.RequestComplete+=new AgentObjects._AgentEvents_RequestCompleteEventHandler(ctl_agent_RequestComplete);
			//ctl_agent.DblClick +=new AgentObjects._AgentEvents_DblClickEventHandler(ctl_agent_DblClick);
			//ctl_agent.Click+=
			//ctl_agent.Command+=new AgentObjects._AgentEvents_CommandEventHandler(ctl_agent_Command);
			//ctl_agent_DblClick

			agent_being_unlading=false;
			agent_being_speaking=false;
			speak_stop_actions=false;
		}

		#endregion

		#region methodes

        public void Show(bool bFast) // bFast : false to show animation
        {
            if (this.obj_char_ex != null)
                this.obj_char_ex.Show(bFast);
        }

		#region load
		public AgentObjects.IAgentCtlCharacterEx load()
		{
			return load("", "unique_agent");
		}
		/// <summary>
		/// used only if you don//t want to use two or more agent at the same time
		/// </summary>
		/// <param name="agent_name"></param>
		/// <param name="agent_folder"></param>
		/// <returns></returns>
		public AgentObjects.IAgentCtlCharacterEx load(string agent_name)
		{
			return load(agent_name, "unique_agent",true);
		}
		public AgentObjects.IAgentCtlCharacterEx load(string agent_name,bool show_agent)
		{
			return load(agent_name, "unique_agent",show_agent);
		}
		public AgentObjects.IAgentCtlCharacterEx load(string agent_name, string key_agent)
		{
			return load(agent_name, key_agent,true);
		}
		public AgentObjects.IAgentCtlCharacterEx load(string agent_name, string key_agent,bool show_agent)
		{
			try
			{
				if (agent_name!="")
				{
                    // make full path if needed
                    if (System.IO.Path.IsPathRooted(agent_name) == false)
                    {
                        string ExeFile = System.Windows.Forms.Application.ExecutablePath;
                        string Dir = System.IO.Path.GetDirectoryName(ExeFile);
                        string Path = System.IO.Path.GetFullPath(System.IO.Path.Combine(Dir, agent_name));
                        agent_name = Path;
                    }

                    if (System.IO.File.Exists(agent_name))
					    ctl_agent.Characters.Load(key_agent, agent_name);
                    else
                        ctl_agent.Characters.Load(key_agent, null);
				}
				else
				{
					ctl_agent.Characters.Load(key_agent, null);
				}
				//Set the object variable for the character.
				this.obj_char_ex = ctl_agent.Characters[key_agent];
                if (show_agent && (obj_char_ex != null)) this.obj_char_ex.Show(false);//false to show animation

				//add pop up menu cmd
                // add_popupmenu();

				//animations
				//play("Greet");
				//play("Pleased");

				agent_unloaded = false;
				return obj_char_ex;
			}
			catch (Exception e)
			{
				if ( (e is System.IO.FileNotFoundException) || (e is System.IO.DirectoryNotFoundException) )
				{
					System.Windows.Forms.MessageBox.Show(null,e.Message + "\r\n" + "Choose another agent file in Startup Options / Agent Options","Error",System.Windows.Forms.MessageBoxButtons.OK ,System.Windows.Forms.MessageBoxIcon.Error);
				}
				else
				{
					System.Windows.Forms.MessageBox.Show(null,e.Message,"Error",System.Windows.Forms.MessageBoxButtons.OK ,System.Windows.Forms.MessageBoxIcon.Exclamation);
				}
				return null;
			}
		}


		#endregion

		#region unload

		public void unload_agent()
		{
			unload_agent("unique_agent");
		}
		public void unload_agent(string key_agent)
		{
			try
			{
				if (obj_char_ex==null) return;
				while(agent_being_speaking)
					System.Windows.Forms.Application.DoEvents();
				if (agent_being_unlading) return;
				obj_char_ex.Stop(null);
				unload_agent_key = key_agent;
				agent_being_unlading=true;
				hide_request = play("hide");
				//see the following action in request completed
				if (hide_request ==null ) real_unload_agent(); //error: hide animation not done, so we force agent to be unloaded
				while (!agent_unloaded)
					System.Windows.Forms.Application.DoEvents();
			}
			catch{}
		}
/*
		private void unload_agent_from_command()
		{
			FormMain.Current_obj_FormMain.timer.Enabled=false;
			unload_agent();
			FormMain.Current_obj_FormMain.Close();
		}
*/

		private void real_unload_agent()
		{
			try
			{
				ctl_agent.Characters.Unload(unload_agent_key);
				obj_char_ex = null;
				hide_request = null;
				agent_unloaded = true;
				agent_being_unlading=false;
			}
			catch{}
		}

		#endregion

		#region play
		/// <summary>
		/// avoid error if agent has not the corresponding animation (return nothing in this case)
		/// </summary>
		/// <param name="animation"></param>
		/// <returns></returns>
		public AgentObjects.IAgentCtlRequest play(string animation)
		{
			try
			{
				if (obj_char_ex==null) return null;
				//if (agent_being_unlading) return null;
				return obj_char_ex.Play(animation);
			}
			catch
			{
				return null;
			}
		}
		#endregion

		#region hide/show ballon

		/// <summary>
		/// 
		/// </summary>
		/// <param name="show">show ballon if true, else hide ballon</param>
		public void show_ballon(bool show)
		{
			if (show) show_ballon();
			else hide_ballon();
		}

		public void show_ballon()
		{
			if (obj_char_ex==null) return;
			// Turn the word balloon on
			obj_char_ex.Balloon.Style = obj_char_ex.Balloon.Style | BalloonOn;

			//style: An integer that represents the balloon//s output style. The style setting is a bit field with bits corresponding to: balloon-on (bit 0), size-to-text (bit 1), auto-hide (bit 2), auto-pace (bit 3), number of characters per lines (bits 16-23), and number of lines (bits 24-31).
			//think: ballon only ex: //obj_char_ex.Think "ballon only"
		}

		public void hide_ballon()
		{
			if (obj_char_ex==null) return;
			obj_char_ex.Balloon.Style = obj_char_ex.Balloon.Style & 254; //(!BalloonOn)
			//obj_char_ex.Speak "No balloon"

			//style: An integer that represents the balloon//s output style. The style setting is a bit field with bits corresponding to: balloon-on (bit 0), size-to-text (bit 1), auto-hide (bit 2), auto-pace (bit 3), number of characters per lines (bits 16-23), and number of lines (bits 24-31).
			//think: ballon only ex: //obj_char_ex.Think "ballon only"
		}

		#endregion

		#region change langage or tts engine

		public void change_tts_engine_from_description(string tts_engine_description)
		{
			change_tts_engine(class_language_and_tts_engine.get_associated_TTSModeID(tts_engine_description));
		}

		public void change_tts_engine(string tts_engine_id)
		{
			try
			{
				if (obj_char_ex==null) return;
				obj_char_ex.TTSModeID = tts_engine_id;
			}
			catch (Exception e)
			{
				if (e is System.Runtime.InteropServices.ExternalException)
				{
					System.Runtime.InteropServices.ExternalException my_e=(System.Runtime.InteropServices.ExternalException)e;
					if (my_e.ErrorCode == -2147213293 )
						System.Windows.Forms.MessageBox.Show(null,e.Message+ "\r\n" + "Please download it at http://www.microsoft.com/msagent/downloads/user.asp","Error",System.Windows.Forms.MessageBoxButtons.OK ,System.Windows.Forms.MessageBoxIcon.Exclamation);
				}
			}
		}

		public void change_language(string language_description)
		{
			change_language(class_language_and_tts_engine.get_associated_languageID(language_description));
		}

		public void change_language(int languageID)
		{
			try
			{
				if (obj_char_ex==null) return;
				obj_char_ex.LanguageID = languageID;
			}
			catch (Exception e)
			{
				if (e is System.Runtime.InteropServices.ExternalException)
				{
					System.Runtime.InteropServices.ExternalException my_e=(System.Runtime.InteropServices.ExternalException)e;
					if (my_e.ErrorCode ==-2147213294 )
						System.Windows.Forms.MessageBox.Show(null,e.Message+ "\r\n" + "Please download it at http://www.microsoft.com/msagent/downloads/user.asp","Error",System.Windows.Forms.MessageBoxButtons.OK ,System.Windows.Forms.MessageBoxIcon.Exclamation);
				}
			}

		}

		#endregion

		#region speak
		public void speak(string text)
		{
			try
			{
				if (text=="") return;
				if (obj_char_ex ==null) return;
				if (agent_being_unlading) return;
				if (speak_stop_actions) obj_char_ex.Stop(null);
				if (!obj_char_ex.Visible) obj_char_ex.Show(false); //agent doesn//t speak if it is hidden
				if (!default_voice)
				{
					if (pittag > 0 ) text = "\\pit=" + pittag + "\\" + text;
					if (spdtag > 0 ) text = "\\spd=" + spdtag + "\\" + text;
					if (chrtag!="") text = "\\chr=" + chrtag + "\\" + text;
					if (ctxtag!="") text = "\\ctx=" + ctxtag + "\\" + text;
				}
				speak_request=obj_char_ex.Speak(text,"");
			}
			catch{}
		}

		public void speak(string text,bool dont_close_if_not_finished)
		{
			agent_being_speaking=dont_close_if_not_finished;//if (dont_close_if_not_finished) agent_being_speaking=true ;
			speak(text);
		}

		#endregion

		#region agent event

		private void ctl_agent_RequestComplete(object Request)
		{
			AgentObjects.IAgentCtlRequest myRequest=(AgentObjects.IAgentCtlRequest)Request;
			if (hide_request!=null)	{if (myRequest.ID == hide_request.ID) {real_unload_agent();}};
			if (speak_request!=null){if (myRequest.ID == speak_request.ID){agent_being_speaking=false;}}
		}


/*
		private void ctl_agent_DblClick(string  CharacterID , short Button ,short Shift ,short x ,short y)
		{
			//speak(System.Windows.Forms.Clipboard.GetDataObject().GetData(System.Windows.Forms.DataFormats.Text).ToString());
			FormMain.Current_obj_FormMain.the_main.show_options();
		}
*/
		#endregion

		#region agent command
/*
		//////////////////////////////////////////////////////////////////
		// adding menu
		private void add_popupmenu()
		{
			obj_char_ex.Commands.RemoveAll();
			// obj_char_ex.Commands.Add("read_clipboard", "R&ead Clipboard","read_clipboard",true,true);
			obj_char_ex.Commands.Add("options", "&Options","options",true,true);
			obj_char_ex.Commands.Add("about", "&About","about",true,true);
			obj_char_ex.Commands.Add("close_agent", "&Close","close",true,true);
		}

		private void ctl_agent_Command(object UserInput)
		{
			AgentObjects.IAgentCtlUserInput myUserInput=(AgentObjects.IAgentCtlUserInput)UserInput;

			switch(myUserInput.Name)
			{

				
				//case "read_clipboard":
				//	speak(System.Windows.Forms.Clipboard.GetDataObject().GetData(System.Windows.Forms.DataFormats.Text).ToString());
				//	break;
				case "options":
					// show form option (avoid multiple instance)
					FormMain.Current_obj_FormMain.the_main.show_options();
					break;
				case "about":
					System.Windows.Forms.MessageBox.Show(null,"Created by Jacquelin POTIER jacquelin.potier@libertysurf.fr","About",System.Windows.Forms.MessageBoxButtons.OK ,System.Windows.Forms.MessageBoxIcon.Information );
					break;
				case "close_agent":
					ThreadStart myThreadStart = new ThreadStart(unload_agent_from_command);
					Thread myThread =new Thread(myThreadStart);
					myThread.Start();
					break;
				default:
					break;
			}
		}
*/		
		#endregion

		#endregion
	}


