// easy_task.cpp: implementation of the easy_task class.
//
//////////////////////////////////////////////////////////////////////

#include "easy_task.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

easy_task::easy_task()
{

}

easy_task::~easy_task()
{

}

int easy_task::get_ITaskScheduler(const char* pc_task_name)
{
    HRESULT hr;
    ///////////////////////////////////////////////////////////////////
    // Call CoInitialize to initialize the COM library and then
    // CoCreateInstance to get the Task Scheduler object.
    ///////////////////////////////////////////////////////////////////

    hr = CoInitialize(NULL);
    if (SUCCEEDED(hr))
    {
        hr = CoCreateInstance(CLSID_CTaskScheduler,
                              NULL,
                              CLSCTX_INPROC_SERVER,
                              IID_ITaskScheduler,
                              (void **) &this->pITS);
        if (FAILED(hr))
        {
            CoUninitialize();
            return 1;
        }
    }
    else
    {
        return 1;
    }
    MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,pc_task_name,strlen(pc_task_name)+1,this->pwszTaskName,sizeof(this->pwszTaskName)/sizeof(this->pwszTaskName[0]) );
    return 0;
}
int easy_task::activate_task()
{
    HRESULT hr;
    ///////////////////////////////////////////////////////////////////
    // Call ITaskScheduler::Activate to get the Task object.
    ///////////////////////////////////////////////////////////////////

    hr = pITS->Activate(this->pwszTaskName,
                      IID_ITask,
                      (IUnknown**) &this->pITask);

    // Release the ITaskScheduler interface.
    pITS->Release();  

    if (FAILED(hr))
    {
        //wprintf(L"Failed calling ITaskScheduler::Activate: ");
        //wprintf(L"error = 0x%x\n",hr);
        CoUninitialize();
        return 1;
    }
    return 0;
}

int easy_task::save_changes()
{

    HRESULT hr;
    hr = this->pITask->QueryInterface(IID_IPersistFile,
                              (void **)&this->pIPersistFile);

    // Release the ITask interface.
    this->pITask->Release();

    if (FAILED(hr))
    {
        CoUninitialize();
        //fprintf(stderr, "Failed calling QueryInterface, error = 0x%x\n",hr);
        return 1;
    }

    hr = this->pIPersistFile->Save(NULL,
                           TRUE);
    // Release the IPersistFile interface.
    this->pIPersistFile->Release();
    if (FAILED(hr))
    {
        CoUninitialize();
        //fprintf(stderr, "Failed calling Save, error = 0x%x\n",hr);
        return 1;
    }
    return 0;
}

int easy_task::create(const char* pc_task_name)
{
    HRESULT hr;
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if(i_ret)
        return i_ret;

    hr = this->pITS->NewWorkItem(this->pwszTaskName,           // Name of task
                     CLSID_CTask,            // Class identifier 
                     IID_ITask,              // Interface identifier
                     (IUnknown**)&this->pITask); // Address of task interface


    this->pITS->Release();                               // Release object
    if (FAILED(hr))
    {
        CoUninitialize();
        //fprintf(stderr, "Failed calling NewWorkItem, error = 0x%x\n",hr);
        return 1;
    }
    i_ret=this->save_changes();
    if(i_ret)
        return i_ret;

    return 0;
}
int easy_task::delete_task(const char* pc_task_name)
{
    HRESULT hr;
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if(i_ret)
        return i_ret;

    hr=pITS->Delete(pwszTaskName);
    if (FAILED(hr))
    {
        //wprintf(L"Failed calling ITaskScheduler::Delete: ");
        //wprintf(L"error = 0x%x\n",hr);
        CoUninitialize();
        return 1;
    } 

    CoUninitialize();
    return 0;
}

bool easy_task::exist(const char* pc_task_name)
{
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if (!i_ret)
    {
        i_ret=this->activate_task();
        CoUninitialize();
    }
    return (i_ret==0);
}


int easy_task::set_application(const char* pc_task_name, const char* pc_full_path_exe)
{
    HRESULT hr;
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if(i_ret)
        return i_ret;
    i_ret=this->activate_task();
    if(i_ret)
        return i_ret;
    ///////////////////////////////////////////////////////////////////
    // Call ITask::SetComment to specify the account name
    // and the account password for Test Task.
    ///////////////////////////////////////////////////////////////////
    //LPCWSTR pwszApplicationName = L"C:\\WINNT40\\notepad.exe";
    WCHAR pwszApplicationName[256];
    MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,pc_full_path_exe,strlen(pc_full_path_exe)+1,pwszApplicationName,sizeof(pwszApplicationName)/sizeof(pwszApplicationName[0]) );
    //LPCWSTR pwszTaskName;
    //pwszTaskName = L"Test Task";
    hr = this->pITask->SetApplicationName(pwszApplicationName);

    if (FAILED(hr))
    {
        //wprintf(L"Failed calling ITask::SetApplicationName: ");
        //wprintf(L"error = 0x%x\n",hr);
        CoUninitialize();
        return 1;
    }
    i_ret=this->save_changes();
    if(i_ret)
        return i_ret;

     //wprintf(L"Set the application name for Test Task.\n");  
    CoUninitialize();
    return 0;
}
int easy_task::set_flags(const char* pc_task_name,DWORD dwflag)// TASK_FLAG_RUN_ONLY_IF_LOGGED_ON 
{
    HRESULT hr;
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if(i_ret)
        return i_ret;
    i_ret=this->activate_task();
    if(i_ret)
        return i_ret;

    hr =this->pITask->SetFlags(dwflag);
    if (FAILED(hr))
    {
        CoUninitialize();
        return 1;
    }

    i_ret=this->save_changes();
    if(i_ret)
        return i_ret;

    CoUninitialize();
    return 0;
}

int easy_task::set_parameters(const char* pc_task_name,char* parameters)
{
    HRESULT hr;
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if(i_ret)
        return i_ret;
    i_ret=this->activate_task();
    if(i_ret)
        return i_ret;

    WCHAR pwszparameters[256];
    MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,parameters,strlen(parameters)+1,pwszparameters,sizeof(pwszparameters)/sizeof(pwszparameters[0]) );
    hr =this->pITask->SetParameters(pwszparameters);
    if (FAILED(hr))
    {
        CoUninitialize();
        return 1;
    }

    i_ret=this->save_changes();
    if(i_ret)
        return i_ret;

    CoUninitialize();
    return 0;
}

// to use current account without givinglogin and pwd:
// set_flags(TaskName,TASK_FLAG_RUN_ONLY_IF_LOGGED_ON);
// set_account_info(TaskName,"",NULL);
int easy_task::set_account_info(const char* pc_task_name,const char* login,const char* pwd)
{
    
    HRESULT hr;
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if(i_ret)
        return i_ret;
    i_ret=this->activate_task();
    if(i_ret)
        return i_ret;


    WCHAR pwszlogin[256];
    MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,login,strlen(login)+1,pwszlogin,sizeof(pwszlogin)/sizeof(pwszlogin[0]) );
    if (pwd !=NULL)
    {
        WCHAR pwszpwd[256];
        MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,pwd,strlen(pwd)+1,pwszpwd,sizeof(pwszpwd)/sizeof(pwszpwd[0]) );
        hr =this->pITask->SetAccountInformation(pwszlogin,pwszpwd);
    }
    else
    {
        hr =this->pITask->SetAccountInformation(pwszlogin,NULL);
    }

    
    if (FAILED(hr))
    {
        CoUninitialize();
        return 1;
    }

    i_ret=this->save_changes();// can fail because of bad login
    if(i_ret)
        return i_ret;

    CoUninitialize();
    return 0; 
}

int easy_task::enable_task_for_logged_user(const char* pc_task_name)
{
    int i_ret;
    i_ret=this->set_flags(pc_task_name,TASK_FLAG_RUN_ONLY_IF_LOGGED_ON);
    if(i_ret)
        return i_ret;
    i_ret=this->set_account_info(pc_task_name,"",NULL);

    return i_ret;
}

int easy_task::set_trigger(const char* pc_task_name,TASK_TRIGGER Trigger)
{
    HRESULT hr;
    int i_ret;
    i_ret=this->get_ITaskScheduler(pc_task_name);
    if(i_ret)
        return i_ret;
    i_ret=this->activate_task();
    if(i_ret)
        return i_ret;
    ///////////////////////////////////////////////////////////////////
    // Call ITask::CreateTrigger to create new trigger.
    ///////////////////////////////////////////////////////////////////

    ITaskTrigger *pITaskTrigger;
    WORD piNewTrigger;
    hr = pITask->CreateTrigger(&piNewTrigger,
                             &pITaskTrigger);
    if (FAILED(hr))
    {
        //wprintf(L"Failed calling ITask::CreatTrigger: ");
        //wprintf(L"error = 0x%x\n",hr);
        CoUninitialize();
        return 1;
    }

    ///////////////////////////////////////////////////////////////////
    // Call ITaskTrigger::SetTrigger to set trigger criteria.
    ///////////////////////////////////////////////////////////////////

    hr = pITaskTrigger->SetTrigger (&Trigger);
    if (FAILED(hr))
    {
        //wprintf(L"Failed calling ITaskTrigger::SetTrigger: ");
        //wprintf(L"error = 0x%x\n",hr);
        CoUninitialize();
        return 1;
    }
    i_ret=this->save_changes();
    if(i_ret)
        return i_ret;

    pITaskTrigger->Release();
    CoUninitialize();
    return 0;
}

int easy_task::set_daily_trigger(const char* pc_task_name,unsigned short start_hour,unsigned short start_minute,unsigned short days_interval)
{
    //////////////////////////////////////////////////////
    // Define TASK_TRIGGER structure. Note that wBeginDay,
    // wBeginMonth, and wBeginYear must be set to a valid 
    // day, month, and year respectively.
    //////////////////////////////////////////////////////

    TASK_TRIGGER pTrigger;
    ZeroMemory(&pTrigger, sizeof (TASK_TRIGGER));

    // Add code to set trigger structure?
    pTrigger.cbTriggerSize = sizeof (TASK_TRIGGER); 
    pTrigger.wBeginDay =EASY_TASK_DAY_BEGIN;                // Required
    pTrigger.wBeginMonth =EASY_TASK_MONTH_BEGIN;            // Required
    pTrigger.wBeginYear =EASY_TASK_YEAR_BEGIN;              // Required
    pTrigger.TriggerType = TASK_TIME_TRIGGER_DAILY;
    pTrigger.Type.Daily.DaysInterval = days_interval;
    pTrigger.wStartHour = start_hour;
    pTrigger.wStartMinute=start_minute;


    return this->set_trigger(pc_task_name,pTrigger);
}

int easy_task::set_daily_trigger(const char* pc_task_name,unsigned short start_hour,unsigned short start_minute)
{
    //////////////////////////////////////////////////////
    // Define TASK_TRIGGER structure. Note that wBeginDay,
    // wBeginMonth, and wBeginYear must be set to a valid 
    // day, month, and year respectively.
    //////////////////////////////////////////////////////

    TASK_TRIGGER pTrigger;
    ZeroMemory(&pTrigger, sizeof (TASK_TRIGGER));

    // Add code to set trigger structure?
    pTrigger.cbTriggerSize = sizeof (TASK_TRIGGER); 
    pTrigger.wBeginDay =EASY_TASK_DAY_BEGIN;                // Required
    pTrigger.wBeginMonth =EASY_TASK_MONTH_BEGIN;            // Required
    pTrigger.wBeginYear =EASY_TASK_YEAR_BEGIN;              // Required
    pTrigger.TriggerType = TASK_TIME_TRIGGER_DAILY;
    pTrigger.Type.Daily.DaysInterval = 1;
    pTrigger.wStartHour = start_hour;
    pTrigger.wStartMinute=start_minute;


    return this->set_trigger(pc_task_name,pTrigger);
}
// days_of_the_month_one_in_bit bit 0 day 1; bit 1 day 2 ...; bit 30 day 31
int easy_task::set_monthlydate_bit_trigger(const char* pc_task_name,unsigned short start_hour,unsigned short start_minute,unsigned long days_of_the_month_one_in_bit)
{
    //////////////////////////////////////////////////////
    // Define TASK_TRIGGER structure. Note that wBeginDay,
    // wBeginMonth, and wBeginYear must be set to a valid 
    // day, month, and year respectively.
    //////////////////////////////////////////////////////

    TASK_TRIGGER pTrigger;
    ZeroMemory(&pTrigger, sizeof (TASK_TRIGGER));

    // Add code to set trigger structure?
    pTrigger.cbTriggerSize = sizeof (TASK_TRIGGER); 
    pTrigger.wBeginDay =EASY_TASK_DAY_BEGIN;                // Required
    pTrigger.wBeginMonth =EASY_TASK_MONTH_BEGIN;            // Required
    pTrigger.wBeginYear =EASY_TASK_YEAR_BEGIN;              // Required
    pTrigger.TriggerType = TASK_TIME_TRIGGER_MONTHLYDATE;
    pTrigger.Type.MonthlyDate.rgfDays = days_of_the_month_one_in_bit;
    pTrigger.Type.MonthlyDate.rgfMonths=TASK_JANUARY | TASK_FEBRUARY | TASK_MARCH | 
        TASK_APRIL | TASK_MAY | TASK_JUNE | TASK_JULY | TASK_AUGUST | 
        TASK_SEPTEMBER | TASK_OCTOBER | TASK_NOVEMBER | TASK_DECEMBER;
    pTrigger.wStartHour = start_hour;
    pTrigger.wStartMinute=start_minute;


    return this->set_trigger(pc_task_name,pTrigger);
}

int easy_task::set_monthlydate_trigger(const char* pc_task_name,unsigned short start_hour,unsigned short start_minute,unsigned char day_of_the_month)
{
    //////////////////////////////////////////////////////
    // Define TASK_TRIGGER structure. Note that wBeginDay,
    // wBeginMonth, and wBeginYear must be set to a valid 
    // day, month, and year respectively.
    //////////////////////////////////////////////////////

    TASK_TRIGGER pTrigger;
    ZeroMemory(&pTrigger, sizeof (TASK_TRIGGER));

    // Add code to set trigger structure?
    pTrigger.cbTriggerSize = sizeof (TASK_TRIGGER); 
    pTrigger.wBeginDay =EASY_TASK_DAY_BEGIN;                // Required
    pTrigger.wBeginMonth =EASY_TASK_MONTH_BEGIN;            // Required
    pTrigger.wBeginYear =EASY_TASK_YEAR_BEGIN;              // Required
    pTrigger.TriggerType = TASK_TIME_TRIGGER_MONTHLYDATE ;
    unsigned long days_of_the_month_one_in_bit=0;
    // days_of_the_month_one_in_bit: bit 0 day 1; bit 1 day 2 ...; bit 30 day 31
    // days_of_the_month_one_in_bit=1<<(32-day_of_the_month+1);
    days_of_the_month_one_in_bit=1<<(sizeof(unsigned long)*8-day_of_the_month+1);
    pTrigger.Type.MonthlyDate.rgfDays = days_of_the_month_one_in_bit;
    pTrigger.Type.MonthlyDate.rgfMonths=TASK_JANUARY | TASK_FEBRUARY | TASK_MARCH | 
        TASK_APRIL | TASK_MAY | TASK_JUNE | TASK_JULY | TASK_AUGUST | 
        TASK_SEPTEMBER | TASK_OCTOBER | TASK_NOVEMBER | TASK_DECEMBER;
    pTrigger.wStartHour = start_hour;
    pTrigger.wStartMinute=start_minute;


    return this->set_trigger(pc_task_name,pTrigger);
}
